/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.cache;

import java.util.Map;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Pull;
import org.objectweb.dream.PullException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.manager.MessageManager;
import org.objectweb.dream.time.GetTimeStamp;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * This class defines a basic cache. A cached message is considered as
 * up-to-date as long as it has not been cached more than <code>timeout</code>
 * milliseconds.
 * <p>
 * <code>timeout</code> can be changed using the
 * <code>CacheAttributeController</code> controller.
 */
public class BasicCacheImpl extends AbstractComponent
    implements
      Pull,
      CacheAttributeController
{

  /** The cache time out. */
  protected long           timeOut = 0;

  /** The message manager client interface. */
  protected MessageManager messageManagerItf;

  /** The GetTimeStamp client interface. */
  protected GetTimeStamp   getTimeStamp;

  /** The Pull client interface. */
  protected Pull           inPull;

  /** The last cached message. */
  protected Message        cachedMessage;

  /** The last time stamp. */
  protected long           lastTimeStamp;

  // -------------------------------------------------------------------------
  // Implementation of the Pull interface
  // -------------------------------------------------------------------------

  /**
   * @see Pull#pull(Map)
   */
  public Message pull(Map context) throws PullException
  {
    long currentTimeStamp = getTimeStamp.getTimeStamp();
    if (currentTimeStamp - lastTimeStamp > timeOut)
    {
      lastTimeStamp = currentTimeStamp;
      messageManagerItf.deleteMessage(cachedMessage);
      cachedMessage = inPull.pull(context);
    }
    return messageManagerItf.duplicateMessage(cachedMessage, true);
  }

  // -------------------------------------------------------------------------
  // Implementation of the CacheAttributeController interface
  // -------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.cache.CacheAttributeController#setTimeOut(long)
   */
  public void setTimeOut(long timeOut)
  {
    this.timeOut = timeOut;
  }

  /**
   * @see org.objectweb.dream.cache.CacheAttributeController#getTimeOut()
   */
  public long getTimeOut()
  {
    return timeOut;
  }

  // -------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // -------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(Pull.IN_PULL_ITF_NAME))
    {
      inPull = (Pull) serverItf;
    }
    else if (clientItfName.equals(GetTimeStamp.ITF_NAME))
    {
      getTimeStamp = (GetTimeStamp) serverItf;
    }
    else if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{Pull.IN_PULL_ITF_NAME, MessageManager.ITF_NAME,
        GetTimeStamp.ITF_NAME};
  }

}