/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): Matthieu Leclercq
 */

package org.objectweb.dream.queue;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.dream.Pull;
import org.objectweb.dream.PullException;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.util.EmptyStringArray;

/**
 * Queue implementation delivering message using a key. Messages are stored in a
 * hashtable. The key corresponding to a message is obtained by calling the
 * <code>getKey</code> method on the Chunk whose name is specified as an
 * initialization parameter.
 */
public class BasicPushPullKeyQueueImpl extends AbstractPushQueueImpl
    implements
      Pull,
      PushPullKeyQueueAttributeController
{

  /**
   * The object used to retreive the key in the context map of the
   * {@link Pull#pull(java.util.Map) }method.
   */
  public static final String KEY_CONTEXT = "key";

  String                     messageNotFoundPolicy;

  String                     keyChunkName;

  Map                        waitingMap  = new HashMap();

  /**
   * @see org.objectweb.dream.Push#push(org.objectweb.dream.message.Message,
   *      java.util.Map)
   */
  public synchronized void push(Message message, Map context)
      throws PushException
  {
    handleOverflow();
    waitingList.add(message);
    this.notify();
  }

  /**
   * @see org.objectweb.dream.Pull#pull(java.util.Map)
   */
  public Message pull(Map context) throws PullException
  {
    Object key = context.get(KEY_CONTEXT);
    if (key == null)
    {
      throw new PullException("Unable to find key object in the context");
    }
    Message msg = (Message) waitingMap.get(key);
    if (msg == null)
    {
      return handlePolicy(key);
    }
    return msg;
  }

  /**
   * Handle "meesage not found" policy, returns <code>null</code> or throw a
   * {@link KeyNotFoundException }.
   * 
   * @param key the not found key
   * @return <code>null</code>
   * @throws KeyNotFoundException if policy is
   *           {@link PushPullKeyQueueAttributeController#EXCEPTION_MESSAGE_NOT_FOUND_POLICY}.
   */
  protected Message handlePolicy(Object key) throws KeyNotFoundException
  {
    if (messageNotFoundPolicy == RETURN_NULL_MESSAGE_NOT_FOUND_POLICY)
    {
      return null;
    }
    else if (messageNotFoundPolicy == EXCEPTION_MESSAGE_NOT_FOUND_POLICY)
    {
      throw new KeyNotFoundException(key);
    }
    return null;
  }

  // ---------------------------------------------------------------------------
  // Implementation of PushQueueAttributeController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.queue.PushPullKeyQueueAttributeController#getMessageNotFoundPolicy()
   */
  public String getMessageNotFoundPolicy()
  {
    return messageNotFoundPolicy;
  }

  /**
   * @see org.objectweb.dream.queue.PushPullKeyQueueAttributeController#setMessageNotFoundPolicy(java.lang.String)
   */
  public void setMessageNotFoundPolicy(String policy)
  {
    // set "message not found" policy attribute reference to interface constant
    // to test policy efficiently.
    if (RETURN_NULL_MESSAGE_NOT_FOUND_POLICY.equals(policy))
    {
      messageNotFoundPolicy = RETURN_NULL_MESSAGE_NOT_FOUND_POLICY;
    }
    else if (EXCEPTION_OVERFLOW_POLICY.equals(policy))
    {
      messageNotFoundPolicy = EXCEPTION_OVERFLOW_POLICY;
    }
    else
    {
      throw new IllegalArgumentException(
          "Unknown \"message not found\" policy : " + policy);
    }
  }

  /**
   * @see org.objectweb.dream.queue.PushPullKeyQueueAttributeController#getKeyName()
   */
  public String getKeyName()
  {
    return keyChunkName;
  }

  /**
   * @see org.objectweb.dream.queue.PushPullKeyQueueAttributeController#setKeyName(java.lang.String)
   */
  public void setKeyName(String name)
  {
    keyChunkName = name;
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return EmptyStringArray.EMPTY_STRING_ARRAY;
  }

}